# encoding: utf-8
# @Author: FAULHABERmotioncontrol
# @Date:   2021-07-01T07:59:41+02:00
# @Email:  FAULHABERMotioncontrol@users.noreply.github.com
# @Project: motioncontrollib
# @Filename: console.py
# @Last modified by:   FAULHABERmotioncontrol
# @Last modified time: 2023-05-25T09:01:13+02:00
# @License: # This module is part of motioncontrollib and is released under
# <see LICENSE file>
# @Copyright: # Copyright (C) 2020-2020 the motioncontrollib authors and contributors
# <see AUTHORS file>


"""
example for using this module in a command line and querying object 1000.00.

if you want a commandline interface use click, argparse or other good libraries
"""
import logging
from collections import namedtuple
from contextlib import contextmanager

import momanlibpy

logging.basicConfig(
    format="[%(levelname)-5.5s]  %(message)s",
    level=logging.DEBUG,
    handlers=[logging.StreamHandler()],
)


@contextmanager
def protocol(
    protocol_dll=momanlibpy.DLL_PATH_CO_USB,
    interface_dll=momanlibpy.DLL_PATH_MC3USB,
):
    """function to automaticall load a device from usb

    (for can use CO_CAN and IXXAT_VCI3 or whatever adapter you have)
    """
    intf = momanlibpy.MomanIntf(interface_dll)
    interfacelist = []
    port_amount, ports, channels, deviceinfos = intf.mmIntfEnumPorts()
    interfaceType = namedtuple("Intf", ["port", "channel", "devinfo"])

    # interfacelist is now [(port, channel, devinfo), ...]
    interfacelist = [
        interfaceType(port=int(x[0]), channel=int(x[1]), devinfo=x[2])
        for x in zip(ports, channels, deviceinfos)
        if len(x[0]) > 0
    ]

    logging.info(f"intfacelist: {interfacelist}")
    if len(interfacelist) == 0:
        logging.info("no devices detected")
        exit(0)
    selected_interface = interfacelist[0]

    if intf.mmIntfIsPortAvailable(selected_interface.port, selected_interface.channel):
        protocol = momanlibpy.MomanProt(protocol_dll=protocol_dll)
        interface_result = protocol.mmProtInitInterface(interface_dll)
        if interface_result is momanlibpy.eMomanprot.eMomanprot_ok:
            logging.debug("open interface was successful")
            baudrates_list = protocol.mmProtSupportedBaudratesList()
            baudrate = baudrates_list[0] if len(baudrates_list) > 0 else 0
            open_result = protocol.mmProtOpenCom(
                selected_interface.port,
                selected_interface.channel,
                baudrate,  # 0 for usb
            )
            if open_result:
                logging.debug("open was successful")
                yield protocol
        else:
            logging.debug(f"open was not successful {interface_result}")
        try:
            # trying to close,
            # as described in EN_7000_05064.pdf 3.1.1 Synchronous communication
            protocol.mmProtCloseCom()
            protocol.mmProtCloseInterface()

        except OSError:
            pass  # we are done here anyways
    else:
        logging.debug("port not available")


def main():
    """main function for this example"""
    with protocol() as proto:
        res = proto.mmProtGetObj(1, 0x1000, 0)
        print("GOBJ 0x1000.00 =", res)


if __name__ == "__main__":
    main()
